// Copyright 2013 Timothy J Fontaine <tjfontaine@gmail.com>
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the 'Software'), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE

var FFI = require('ffi-napi'),
    ArrayType = require('ref-array-napi'),
    Struct = require('ref-struct-napi'),
    ref = require('ref-napi');
var voidPtr = ref.refType(ref.types.void);
exports.CONSTANTS = {
    CXDiagnosticSeverity: {
        '0': 'CXDiagnostic_Ignored',
        '1': 'CXDiagnostic_Note',
        '2': 'CXDiagnostic_Warning',
        '3': 'CXDiagnostic_Error',
        '4': 'CXDiagnostic_Fatal',
        CXDiagnostic_Ignored: 0,
        CXDiagnostic_Note: 1,
        CXDiagnostic_Warning: 2,
        CXDiagnostic_Error: 3,
        CXDiagnostic_Fatal: 4
    },
    CXTUResourceUsageKind: {
        '1': 'CXTUResourceUsage_First',
        '2': 'CXTUResourceUsage_Identifiers',
        '3': 'CXTUResourceUsage_Selectors',
        '4': 'CXTUResourceUsage_GlobalCompletionResults',
        '5': 'CXTUResourceUsage_SourceManagerContentCache',
        '6': 'CXTUResourceUsage_AST_SideTables',
        '7': 'CXTUResourceUsage_SourceManager_Membuffer_Malloc',
        '8': 'CXTUResourceUsage_SourceManager_Membuffer_MMap',
        '9': 'CXTUResourceUsage_ExternalASTSource_Membuffer_Malloc',
        '10': 'CXTUResourceUsage_ExternalASTSource_Membuffer_MMap',
        '11': 'CXTUResourceUsage_Preprocessor',
        '12': 'CXTUResourceUsage_PreprocessingRecord',
        '13': 'CXTUResourceUsage_SourceManager_DataStructures',
        '14': 'CXTUResourceUsage_Last',
        CXTUResourceUsage_AST: 1,
        CXTUResourceUsage_Identifiers: 2,
        CXTUResourceUsage_Selectors: 3,
        CXTUResourceUsage_GlobalCompletionResults: 4,
        CXTUResourceUsage_SourceManagerContentCache: 5,
        CXTUResourceUsage_AST_SideTables: 6,
        CXTUResourceUsage_SourceManager_Membuffer_Malloc: 7,
        CXTUResourceUsage_SourceManager_Membuffer_MMap: 8,
        CXTUResourceUsage_ExternalASTSource_Membuffer_Malloc: 9,
        CXTUResourceUsage_ExternalASTSource_Membuffer_MMap: 10,
        CXTUResourceUsage_Preprocessor: 11,
        CXTUResourceUsage_PreprocessingRecord: 12,
        CXTUResourceUsage_SourceManager_DataStructures: 13,
        CXTUResourceUsage_Preprocessor_HeaderSearch: 14,
        CXTUResourceUsage_MEMORY_IN_BYTES_BEGIN: 1,
        CXTUResourceUsage_MEMORY_IN_BYTES_END: 14,
        CXTUResourceUsage_First: 1,
        CXTUResourceUsage_Last: 14
    },
    CXCursorKind: {
        '1': 'CXCursor_FirstDecl',
        '2': 'CXCursor_StructDecl',
        '3': 'CXCursor_UnionDecl',
        '4': 'CXCursor_ClassDecl',
        '5': 'CXCursor_EnumDecl',
        '6': 'CXCursor_FieldDecl',
        '7': 'CXCursor_EnumConstantDecl',
        '8': 'CXCursor_FunctionDecl',
        '9': 'CXCursor_VarDecl',
        '10': 'CXCursor_ParmDecl',
        '11': 'CXCursor_ObjCInterfaceDecl',
        '12': 'CXCursor_ObjCCategoryDecl',
        '13': 'CXCursor_ObjCProtocolDecl',
        '14': 'CXCursor_ObjCPropertyDecl',
        '15': 'CXCursor_ObjCIvarDecl',
        '16': 'CXCursor_ObjCInstanceMethodDecl',
        '17': 'CXCursor_ObjCClassMethodDecl',
        '18': 'CXCursor_ObjCImplementationDecl',
        '19': 'CXCursor_ObjCCategoryImplDecl',
        '20': 'CXCursor_TypedefDecl',
        '21': 'CXCursor_CXXMethod',
        '22': 'CXCursor_Namespace',
        '23': 'CXCursor_LinkageSpec',
        '24': 'CXCursor_Constructor',
        '25': 'CXCursor_Destructor',
        '26': 'CXCursor_ConversionFunction',
        '27': 'CXCursor_TemplateTypeParameter',
        '28': 'CXCursor_NonTypeTemplateParameter',
        '29': 'CXCursor_TemplateTemplateParameter',
        '30': 'CXCursor_FunctionTemplate',
        '31': 'CXCursor_ClassTemplate',
        '32': 'CXCursor_ClassTemplatePartialSpecialization',
        '33': 'CXCursor_NamespaceAlias',
        '34': 'CXCursor_UsingDirective',
        '35': 'CXCursor_UsingDeclaration',
        '36': 'CXCursor_TypeAliasDecl',
        '37': 'CXCursor_ObjCSynthesizeDecl',
        '38': 'CXCursor_ObjCDynamicDecl',
        '39': 'CXCursor_LastDecl',
        '40': 'CXCursor_ObjCSuperClassRef',
        '41': 'CXCursor_ObjCProtocolRef',
        '42': 'CXCursor_ObjCClassRef',
        '43': 'CXCursor_TypeRef',
        '44': 'CXCursor_CXXBaseSpecifier',
        '45': 'CXCursor_TemplateRef',
        '46': 'CXCursor_NamespaceRef',
        '47': 'CXCursor_MemberRef',
        '48': 'CXCursor_LabelRef',
        '49': 'CXCursor_OverloadedDeclRef',
        '50': 'CXCursor_LastRef',
        '70': 'CXCursor_InvalidFile',
        '71': 'CXCursor_NoDeclFound',
        '72': 'CXCursor_NotImplemented',
        '73': 'CXCursor_LastInvalid',
        '100': 'CXCursor_UnexposedExpr',
        '101': 'CXCursor_DeclRefExpr',
        '102': 'CXCursor_MemberRefExpr',
        '103': 'CXCursor_CallExpr',
        '104': 'CXCursor_ObjCMessageExpr',
        '105': 'CXCursor_BlockExpr',
        '106': 'CXCursor_IntegerLiteral',
        '107': 'CXCursor_FloatingLiteral',
        '108': 'CXCursor_ImaginaryLiteral',
        '109': 'CXCursor_StringLiteral',
        '110': 'CXCursor_CharacterLiteral',
        '111': 'CXCursor_ParenExpr',
        '112': 'CXCursor_UnaryOperator',
        '113': 'CXCursor_ArraySubscriptExpr',
        '114': 'CXCursor_BinaryOperator',
        '115': 'CXCursor_CompoundAssignOperator',
        '116': 'CXCursor_ConditionalOperator',
        '117': 'CXCursor_CStyleCastExpr',
        '118': 'CXCursor_CompoundLiteralExpr',
        '119': 'CXCursor_InitListExpr',
        '120': 'CXCursor_AddrLabelExpr',
        '121': 'CXCursor_StmtExpr',
        '122': 'CXCursor_GenericSelectionExpr',
        '123': 'CXCursor_GNUNullExpr',
        '124': 'CXCursor_CXXStaticCastExpr',
        '125': 'CXCursor_CXXDynamicCastExpr',
        '126': 'CXCursor_CXXReinterpretCastExpr',
        '127': 'CXCursor_CXXConstCastExpr',
        '128': 'CXCursor_CXXFunctionalCastExpr',
        '129': 'CXCursor_CXXTypeidExpr',
        '130': 'CXCursor_CXXBoolLiteralExpr',
        '131': 'CXCursor_CXXNullPtrLiteralExpr',
        '132': 'CXCursor_CXXThisExpr',
        '133': 'CXCursor_CXXThrowExpr',
        '134': 'CXCursor_CXXNewExpr',
        '135': 'CXCursor_CXXDeleteExpr',
        '136': 'CXCursor_UnaryExpr',
        '137': 'CXCursor_ObjCStringLiteral',
        '138': 'CXCursor_ObjCEncodeExpr',
        '139': 'CXCursor_ObjCSelectorExpr',
        '140': 'CXCursor_ObjCProtocolExpr',
        '141': 'CXCursor_ObjCBridgedCastExpr',
        '142': 'CXCursor_PackExpansionExpr',
        '143': 'CXCursor_SizeOfPackExpr',
        '144': 'CXCursor_LambdaExpr',
        '145': 'CXCursor_LastExpr',
        '200': 'CXCursor_UnexposedStmt',
        '201': 'CXCursor_LabelStmt',
        '202': 'CXCursor_CompoundStmt',
        '203': 'CXCursor_CaseStmt',
        '204': 'CXCursor_DefaultStmt',
        '205': 'CXCursor_IfStmt',
        '206': 'CXCursor_SwitchStmt',
        '207': 'CXCursor_WhileStmt',
        '208': 'CXCursor_DoStmt',
        '209': 'CXCursor_ForStmt',
        '210': 'CXCursor_GotoStmt',
        '211': 'CXCursor_IndirectGotoStmt',
        '212': 'CXCursor_ContinueStmt',
        '213': 'CXCursor_BreakStmt',
        '214': 'CXCursor_ReturnStmt',
        '215': 'CXCursor_AsmStmt',
        '216': 'CXCursor_ObjCAtTryStmt',
        '217': 'CXCursor_ObjCAtCatchStmt',
        '218': 'CXCursor_ObjCAtFinallyStmt',
        '219': 'CXCursor_ObjCAtThrowStmt',
        '220': 'CXCursor_ObjCAtSynchronizedStmt',
        '221': 'CXCursor_ObjCAutoreleasePoolStmt',
        '222': 'CXCursor_ObjCForCollectionStmt',
        '223': 'CXCursor_CXXCatchStmt',
        '224': 'CXCursor_CXXTryStmt',
        '225': 'CXCursor_CXXForRangeStmt',
        '226': 'CXCursor_SEHTryStmt',
        '227': 'CXCursor_SEHExceptStmt',
        '228': 'CXCursor_SEHFinallyStmt',
        '229': 'CXCursor_MSAsmStmt',
        '230': 'CXCursor_NullStmt',
        '231': 'CXCursor_LastStmt',
        '300': 'CXCursor_TranslationUnit',
        '400': 'CXCursor_UnexposedAttr',
        '401': 'CXCursor_IBActionAttr',
        '402': 'CXCursor_IBOutletAttr',
        '403': 'CXCursor_IBOutletCollectionAttr',
        '404': 'CXCursor_CXXFinalAttr',
        '405': 'CXCursor_CXXOverrideAttr',
        '406': 'CXCursor_AnnotateAttr',
        '407': 'CXCursor_LastAttr',
        '500': 'CXCursor_FirstPreprocessing',
        '501': 'CXCursor_MacroDefinition',
        '502': 'CXCursor_MacroInstantiation',
        '503': 'CXCursor_LastPreprocessing',
        '600': 'CXCursor_LastExtraDecl',
        CXCursor_UnexposedDecl: 1,
        CXCursor_StructDecl: 2,
        CXCursor_UnionDecl: 3,
        CXCursor_ClassDecl: 4,
        CXCursor_EnumDecl: 5,
        CXCursor_FieldDecl: 6,
        CXCursor_EnumConstantDecl: 7,
        CXCursor_FunctionDecl: 8,
        CXCursor_VarDecl: 9,
        CXCursor_ParmDecl: 10,
        CXCursor_ObjCInterfaceDecl: 11,
        CXCursor_ObjCCategoryDecl: 12,
        CXCursor_ObjCProtocolDecl: 13,
        CXCursor_ObjCPropertyDecl: 14,
        CXCursor_ObjCIvarDecl: 15,
        CXCursor_ObjCInstanceMethodDecl: 16,
        CXCursor_ObjCClassMethodDecl: 17,
        CXCursor_ObjCImplementationDecl: 18,
        CXCursor_ObjCCategoryImplDecl: 19,
        CXCursor_TypedefDecl: 20,
        CXCursor_CXXMethod: 21,
        CXCursor_Namespace: 22,
        CXCursor_LinkageSpec: 23,
        CXCursor_Constructor: 24,
        CXCursor_Destructor: 25,
        CXCursor_ConversionFunction: 26,
        CXCursor_TemplateTypeParameter: 27,
        CXCursor_NonTypeTemplateParameter: 28,
        CXCursor_TemplateTemplateParameter: 29,
        CXCursor_FunctionTemplate: 30,
        CXCursor_ClassTemplate: 31,
        CXCursor_ClassTemplatePartialSpecialization: 32,
        CXCursor_NamespaceAlias: 33,
        CXCursor_UsingDirective: 34,
        CXCursor_UsingDeclaration: 35,
        CXCursor_TypeAliasDecl: 36,
        CXCursor_ObjCSynthesizeDecl: 37,
        CXCursor_ObjCDynamicDecl: 38,
        CXCursor_CXXAccessSpecifier: 39,
        CXCursor_FirstDecl: 1,
        CXCursor_LastDecl: 39,
        CXCursor_FirstRef: 40,
        CXCursor_ObjCSuperClassRef: 40,
        CXCursor_ObjCProtocolRef: 41,
        CXCursor_ObjCClassRef: 42,
        CXCursor_TypeRef: 43,
        CXCursor_CXXBaseSpecifier: 44,
        CXCursor_TemplateRef: 45,
        CXCursor_NamespaceRef: 46,
        CXCursor_MemberRef: 47,
        CXCursor_LabelRef: 48,
        CXCursor_OverloadedDeclRef: 49,
        CXCursor_VariableRef: 50,
        CXCursor_LastRef: 50,
        CXCursor_FirstInvalid: 70,
        CXCursor_InvalidFile: 70,
        CXCursor_NoDeclFound: 71,
        CXCursor_NotImplemented: 72,
        CXCursor_InvalidCode: 73,
        CXCursor_LastInvalid: 73,
        CXCursor_FirstExpr: 100,
        CXCursor_UnexposedExpr: 100,
        CXCursor_DeclRefExpr: 101,
        CXCursor_MemberRefExpr: 102,
        CXCursor_CallExpr: 103,
        CXCursor_ObjCMessageExpr: 104,
        CXCursor_BlockExpr: 105,
        CXCursor_IntegerLiteral: 106,
        CXCursor_FloatingLiteral: 107,
        CXCursor_ImaginaryLiteral: 108,
        CXCursor_StringLiteral: 109,
        CXCursor_CharacterLiteral: 110,
        CXCursor_ParenExpr: 111,
        CXCursor_UnaryOperator: 112,
        CXCursor_ArraySubscriptExpr: 113,
        CXCursor_BinaryOperator: 114,
        CXCursor_CompoundAssignOperator: 115,
        CXCursor_ConditionalOperator: 116,
        CXCursor_CStyleCastExpr: 117,
        CXCursor_CompoundLiteralExpr: 118,
        CXCursor_InitListExpr: 119,
        CXCursor_AddrLabelExpr: 120,
        CXCursor_StmtExpr: 121,
        CXCursor_GenericSelectionExpr: 122,
        CXCursor_GNUNullExpr: 123,
        CXCursor_CXXStaticCastExpr: 124,
        CXCursor_CXXDynamicCastExpr: 125,
        CXCursor_CXXReinterpretCastExpr: 126,
        CXCursor_CXXConstCastExpr: 127,
        CXCursor_CXXFunctionalCastExpr: 128,
        CXCursor_CXXTypeidExpr: 129,
        CXCursor_CXXBoolLiteralExpr: 130,
        CXCursor_CXXNullPtrLiteralExpr: 131,
        CXCursor_CXXThisExpr: 132,
        CXCursor_CXXThrowExpr: 133,
        CXCursor_CXXNewExpr: 134,
        CXCursor_CXXDeleteExpr: 135,
        CXCursor_UnaryExpr: 136,
        CXCursor_ObjCStringLiteral: 137,
        CXCursor_ObjCEncodeExpr: 138,
        CXCursor_ObjCSelectorExpr: 139,
        CXCursor_ObjCProtocolExpr: 140,
        CXCursor_ObjCBridgedCastExpr: 141,
        CXCursor_PackExpansionExpr: 142,
        CXCursor_SizeOfPackExpr: 143,
        CXCursor_LambdaExpr: 144,
        CXCursor_ObjCBoolLiteralExpr: 145,
        CXCursor_LastExpr: 145,
        CXCursor_FirstStmt: 200,
        CXCursor_UnexposedStmt: 200,
        CXCursor_LabelStmt: 201,
        CXCursor_CompoundStmt: 202,
        CXCursor_CaseStmt: 203,
        CXCursor_DefaultStmt: 204,
        CXCursor_IfStmt: 205,
        CXCursor_SwitchStmt: 206,
        CXCursor_WhileStmt: 207,
        CXCursor_DoStmt: 208,
        CXCursor_ForStmt: 209,
        CXCursor_GotoStmt: 210,
        CXCursor_IndirectGotoStmt: 211,
        CXCursor_ContinueStmt: 212,
        CXCursor_BreakStmt: 213,
        CXCursor_ReturnStmt: 214,
        CXCursor_GCCAsmStmt: 215,
        CXCursor_AsmStmt: 215,
        CXCursor_ObjCAtTryStmt: 216,
        CXCursor_ObjCAtCatchStmt: 217,
        CXCursor_ObjCAtFinallyStmt: 218,
        CXCursor_ObjCAtThrowStmt: 219,
        CXCursor_ObjCAtSynchronizedStmt: 220,
        CXCursor_ObjCAutoreleasePoolStmt: 221,
        CXCursor_ObjCForCollectionStmt: 222,
        CXCursor_CXXCatchStmt: 223,
        CXCursor_CXXTryStmt: 224,
        CXCursor_CXXForRangeStmt: 225,
        CXCursor_SEHTryStmt: 226,
        CXCursor_SEHExceptStmt: 227,
        CXCursor_SEHFinallyStmt: 228,
        CXCursor_MSAsmStmt: 229,
        CXCursor_NullStmt: 230,
        CXCursor_DeclStmt: 231,
        CXCursor_LastStmt: 231,
        CXCursor_TranslationUnit: 300,
        CXCursor_FirstAttr: 400,
        CXCursor_UnexposedAttr: 400,
        CXCursor_IBActionAttr: 401,
        CXCursor_IBOutletAttr: 402,
        CXCursor_IBOutletCollectionAttr: 403,
        CXCursor_CXXFinalAttr: 404,
        CXCursor_CXXOverrideAttr: 405,
        CXCursor_AnnotateAttr: 406,
        CXCursor_AsmLabelAttr: 407,
        CXCursor_LastAttr: 407,
        CXCursor_PreprocessingDirective: 500,
        CXCursor_MacroDefinition: 501,
        CXCursor_MacroExpansion: 502,
        CXCursor_MacroInstantiation: 502,
        CXCursor_InclusionDirective: 503,
        CXCursor_FirstPreprocessing: 500,
        CXCursor_LastPreprocessing: 503,
        CXCursor_ModuleImportDecl: 600,
        CXCursor_FirstExtraDecl: 600,
        CXCursor_LastExtraDecl: 600
    },
    CXLinkageKind: {
        '0': 'CXLinkage_Invalid',
        '1': 'CXLinkage_NoLinkage',
        '2': 'CXLinkage_Internal',
        '3': 'CXLinkage_UniqueExternal',
        '4': 'CXLinkage_External',
        CXLinkage_Invalid: 0,
        CXLinkage_NoLinkage: 1,
        CXLinkage_Internal: 2,
        CXLinkage_UniqueExternal: 3,
        CXLinkage_External: 4
    },
    CXAvailabilityKind: {
        '0': 'CXAvailability_Available',
        '1': 'CXAvailability_Deprecated',
        '2': 'CXAvailability_NotAvailable',
        '3': 'CXAvailability_NotAccessible',
        CXAvailability_Available: 0,
        CXAvailability_Deprecated: 1,
        CXAvailability_NotAvailable: 2,
        CXAvailability_NotAccessible: 3
    },
    CXLanguageKind: {
        '0': 'CXLanguage_Invalid',
        '1': 'CXLanguage_C',
        '2': 'CXLanguage_ObjC',
        '3': 'CXLanguage_CPlusPlus',
        CXLanguage_Invalid: 0,
        CXLanguage_C: 1,
        CXLanguage_ObjC: 2,
        CXLanguage_CPlusPlus: 3
    },
    CXTypeKind: {
        '0': 'CXType_Invalid',
        '1': 'CXType_Unexposed',
        '2': 'CXType_FirstBuiltin',
        '3': 'CXType_Bool',
        '4': 'CXType_Char_U',
        '5': 'CXType_UChar',
        '6': 'CXType_Char16',
        '7': 'CXType_Char32',
        '8': 'CXType_UShort',
        '9': 'CXType_UInt',
        '10': 'CXType_ULong',
        '11': 'CXType_ULongLong',
        '12': 'CXType_UInt128',
        '13': 'CXType_Char_S',
        '14': 'CXType_SChar',
        '15': 'CXType_WChar',
        '16': 'CXType_Short',
        '17': 'CXType_Int',
        '18': 'CXType_Long',
        '19': 'CXType_LongLong',
        '20': 'CXType_Int128',
        '21': 'CXType_Float',
        '22': 'CXType_Double',
        '23': 'CXType_LongDouble',
        '24': 'CXType_NullPtr',
        '25': 'CXType_Overload',
        '26': 'CXType_Dependent',
        '27': 'CXType_ObjCId',
        '28': 'CXType_ObjCClass',
        '29': 'CXType_LastBuiltin',
        '100': 'CXType_Complex',
        '101': 'CXType_Pointer',
        '102': 'CXType_BlockPointer',
        '103': 'CXType_LValueReference',
        '104': 'CXType_RValueReference',
        '105': 'CXType_Record',
        '106': 'CXType_Enum',
        '107': 'CXType_Typedef',
        '108': 'CXType_ObjCInterface',
        '109': 'CXType_ObjCObjectPointer',
        '110': 'CXType_FunctionNoProto',
        '111': 'CXType_FunctionProto',
        '112': 'CXType_ConstantArray',
        '113': 'CXType_Vector',
        CXType_Invalid: 0,
        CXType_Unexposed: 1,
        CXType_Void: 2,
        CXType_Bool: 3,
        CXType_Char_U: 4,
        CXType_UChar: 5,
        CXType_Char16: 6,
        CXType_Char32: 7,
        CXType_UShort: 8,
        CXType_UInt: 9,
        CXType_ULong: 10,
        CXType_ULongLong: 11,
        CXType_UInt128: 12,
        CXType_Char_S: 13,
        CXType_SChar: 14,
        CXType_WChar: 15,
        CXType_Short: 16,
        CXType_Int: 17,
        CXType_Long: 18,
        CXType_LongLong: 19,
        CXType_Int128: 20,
        CXType_Float: 21,
        CXType_Double: 22,
        CXType_LongDouble: 23,
        CXType_NullPtr: 24,
        CXType_Overload: 25,
        CXType_Dependent: 26,
        CXType_ObjCId: 27,
        CXType_ObjCClass: 28,
        CXType_ObjCSel: 29,
        CXType_FirstBuiltin: 2,
        CXType_LastBuiltin: 29,
        CXType_Complex: 100,
        CXType_Pointer: 101,
        CXType_BlockPointer: 102,
        CXType_LValueReference: 103,
        CXType_RValueReference: 104,
        CXType_Record: 105,
        CXType_Enum: 106,
        CXType_Typedef: 107,
        CXType_ObjCInterface: 108,
        CXType_ObjCObjectPointer: 109,
        CXType_FunctionNoProto: 110,
        CXType_FunctionProto: 111,
        CXType_ConstantArray: 112,
        CXType_Vector: 113
    },
    CXCallingConv: {
        '0': 'CXCallingConv_Default',
        '1': 'CXCallingConv_C',
        '2': 'CXCallingConv_X86StdCall',
        '3': 'CXCallingConv_X86FastCall',
        '4': 'CXCallingConv_X86ThisCall',
        '5': 'CXCallingConv_X86Pascal',
        '6': 'CXCallingConv_AAPCS',
        '7': 'CXCallingConv_AAPCS_VFP',
        '8': 'CXCallingConv_PnaclCall',
        '100': 'CXCallingConv_Invalid',
        '200': 'CXCallingConv_Unexposed',
        CXCallingConv_Default: 0,
        CXCallingConv_C: 1,
        CXCallingConv_X86StdCall: 2,
        CXCallingConv_X86FastCall: 3,
        CXCallingConv_X86ThisCall: 4,
        CXCallingConv_X86Pascal: 5,
        CXCallingConv_AAPCS: 6,
        CXCallingConv_AAPCS_VFP: 7,
        CXCallingConv_PnaclCall: 8,
        CXCallingConv_Invalid: 100,
        CXCallingConv_Unexposed: 200
    },
    CX_CXXAccessSpecifier: {
        '0': 'CX_CXXInvalidAccessSpecifier',
        '1': 'CX_CXXPublic',
        '2': 'CX_CXXProtected',
        '3': 'CX_CXXPrivate',
        CX_CXXInvalidAccessSpecifier: 0,
        CX_CXXPublic: 1,
        CX_CXXProtected: 2,
        CX_CXXPrivate: 3
    },
    CXChildVisitResult: {
        '0': 'CXChildVisit_Break',
        '1': 'CXChildVisit_Continue',
        '2': 'CXChildVisit_Recurse',
        CXChildVisit_Break: 0,
        CXChildVisit_Continue: 1,
        CXChildVisit_Recurse: 2
    },
    CXCommentKind: {
        '0': 'CXComment_Null',
        '1': 'CXComment_Text',
        '2': 'CXComment_InlineCommand',
        '3': 'CXComment_HTMLStartTag',
        '4': 'CXComment_HTMLEndTag',
        '5': 'CXComment_Paragraph',
        '6': 'CXComment_BlockCommand',
        '7': 'CXComment_ParamCommand',
        '8': 'CXComment_TParamCommand',
        '9': 'CXComment_VerbatimBlockCommand',
        '10': 'CXComment_VerbatimBlockLine',
        '11': 'CXComment_VerbatimLine',
        '12': 'CXComment_FullComment',
        CXComment_Null: 0,
        CXComment_Text: 1,
        CXComment_InlineCommand: 2,
        CXComment_HTMLStartTag: 3,
        CXComment_HTMLEndTag: 4,
        CXComment_Paragraph: 5,
        CXComment_BlockCommand: 6,
        CXComment_ParamCommand: 7,
        CXComment_TParamCommand: 8,
        CXComment_VerbatimBlockCommand: 9,
        CXComment_VerbatimBlockLine: 10,
        CXComment_VerbatimLine: 11,
        CXComment_FullComment: 12
    },
    CXCommentInlineCommandRenderKind: {
        '0': 'CXCommentInlineCommandRenderKind_Normal',
        '1': 'CXCommentInlineCommandRenderKind_Bold',
        '2': 'CXCommentInlineCommandRenderKind_Monospaced',
        '3': 'CXCommentInlineCommandRenderKind_Emphasized',
        CXCommentInlineCommandRenderKind_Normal: 0,
        CXCommentInlineCommandRenderKind_Bold: 1,
        CXCommentInlineCommandRenderKind_Monospaced: 2,
        CXCommentInlineCommandRenderKind_Emphasized: 3
    },
    CXCommentParamPassDirection: {
        '0': 'CXCommentParamPassDirection_In',
        '1': 'CXCommentParamPassDirection_Out',
        '2': 'CXCommentParamPassDirection_InOut',
        CXCommentParamPassDirection_In: 0,
        CXCommentParamPassDirection_Out: 1,
        CXCommentParamPassDirection_InOut: 2
    },
    CXTokenKind: {
        '0': 'CXToken_Punctuation',
        '1': 'CXToken_Keyword',
        '2': 'CXToken_Identifier',
        '3': 'CXToken_Literal',
        '4': 'CXToken_Comment',
        CXToken_Punctuation: 0,
        CXToken_Keyword: 1,
        CXToken_Identifier: 2,
        CXToken_Literal: 3,
        CXToken_Comment: 4
    },
    CXCompletionChunkKind: {
        '0': 'CXCompletionChunk_Optional',
        '1': 'CXCompletionChunk_TypedText',
        '2': 'CXCompletionChunk_Text',
        '3': 'CXCompletionChunk_Placeholder',
        '4': 'CXCompletionChunk_Informative',
        '5': 'CXCompletionChunk_CurrentParameter',
        '6': 'CXCompletionChunk_LeftParen',
        '7': 'CXCompletionChunk_RightParen',
        '8': 'CXCompletionChunk_LeftBracket',
        '9': 'CXCompletionChunk_RightBracket',
        '10': 'CXCompletionChunk_LeftBrace',
        '11': 'CXCompletionChunk_RightBrace',
        '12': 'CXCompletionChunk_LeftAngle',
        '13': 'CXCompletionChunk_RightAngle',
        '14': 'CXCompletionChunk_Comma',
        '15': 'CXCompletionChunk_ResultType',
        '16': 'CXCompletionChunk_Colon',
        '17': 'CXCompletionChunk_SemiColon',
        '18': 'CXCompletionChunk_Equal',
        '19': 'CXCompletionChunk_HorizontalSpace',
        '20': 'CXCompletionChunk_VerticalSpace',
        CXCompletionChunk_Optional: 0,
        CXCompletionChunk_TypedText: 1,
        CXCompletionChunk_Text: 2,
        CXCompletionChunk_Placeholder: 3,
        CXCompletionChunk_Informative: 4,
        CXCompletionChunk_CurrentParameter: 5,
        CXCompletionChunk_LeftParen: 6,
        CXCompletionChunk_RightParen: 7,
        CXCompletionChunk_LeftBracket: 8,
        CXCompletionChunk_RightBracket: 9,
        CXCompletionChunk_LeftBrace: 10,
        CXCompletionChunk_RightBrace: 11,
        CXCompletionChunk_LeftAngle: 12,
        CXCompletionChunk_RightAngle: 13,
        CXCompletionChunk_Comma: 14,
        CXCompletionChunk_ResultType: 15,
        CXCompletionChunk_Colon: 16,
        CXCompletionChunk_SemiColon: 17,
        CXCompletionChunk_Equal: 18,
        CXCompletionChunk_HorizontalSpace: 19,
        CXCompletionChunk_VerticalSpace: 20
    },
    CXVisitorResult: {
        '0': 'CXVisit_Break',
        '1': 'CXVisit_Continue',
        CXVisit_Break: 0,
        CXVisit_Continue: 1
    },
    '': {
        '0': 'CXIdxAttr_Unexposed',
        '1': 'CXIdxAttr_IBAction',
        '2': 'CXIdxAttr_IBOutlet',
        '3': 'CXIdxAttr_IBOutletCollection',
        '4': 'CXIdxEntity_Field',
        '5': 'CXIdxEntity_EnumConstant',
        '6': 'CXIdxEntity_ObjCClass',
        '7': 'CXIdxEntity_ObjCProtocol',
        '8': 'CXIdxEntity_ObjCCategory',
        '9': 'CXIdxEntity_ObjCInstanceMethod',
        '10': 'CXIdxEntity_ObjCClassMethod',
        '11': 'CXIdxEntity_ObjCProperty',
        '12': 'CXIdxEntity_ObjCIvar',
        '13': 'CXIdxEntity_Enum',
        '14': 'CXIdxEntity_Struct',
        '15': 'CXIdxEntity_Union',
        '16': 'CXIdxEntity_CXXClass',
        '17': 'CXIdxEntity_CXXNamespace',
        '18': 'CXIdxEntity_CXXNamespaceAlias',
        '19': 'CXIdxEntity_CXXStaticVariable',
        '20': 'CXIdxEntity_CXXStaticMethod',
        '21': 'CXIdxEntity_CXXInstanceMethod',
        '22': 'CXIdxEntity_CXXConstructor',
        '23': 'CXIdxEntity_CXXDestructor',
        '24': 'CXIdxEntity_CXXConversionFunction',
        '25': 'CXIdxEntity_CXXTypeAlias',
        '26': 'CXIdxEntity_CXXInterface',
        CXIdxEntity_Unexposed: 0,
        CXIdxEntity_Typedef: 1,
        CXIdxEntity_Function: 2,
        CXIdxEntity_Variable: 3,
        CXIdxEntity_Field: 4,
        CXIdxEntity_EnumConstant: 5,
        CXIdxEntity_ObjCClass: 6,
        CXIdxEntity_ObjCProtocol: 7,
        CXIdxEntity_ObjCCategory: 8,
        CXIdxEntity_ObjCInstanceMethod: 9,
        CXIdxEntity_ObjCClassMethod: 10,
        CXIdxEntity_ObjCProperty: 11,
        CXIdxEntity_ObjCIvar: 12,
        CXIdxEntity_Enum: 13,
        CXIdxEntity_Struct: 14,
        CXIdxEntity_Union: 15,
        CXIdxEntity_CXXClass: 16,
        CXIdxEntity_CXXNamespace: 17,
        CXIdxEntity_CXXNamespaceAlias: 18,
        CXIdxEntity_CXXStaticVariable: 19,
        CXIdxEntity_CXXStaticMethod: 20,
        CXIdxEntity_CXXInstanceMethod: 21,
        CXIdxEntity_CXXConstructor: 22,
        CXIdxEntity_CXXDestructor: 23,
        CXIdxEntity_CXXConversionFunction: 24,
        CXIdxEntity_CXXTypeAlias: 25,
        CXIdxEntity_CXXInterface: 26,
        CXIdxEntity_NonTemplate: 0,
        CXIdxEntity_Template: 1,
        CXIdxEntity_TemplatePartialSpecialization: 2,
        CXIdxEntity_TemplateSpecialization: 3,
        CXIdxEntityLang_None: 0,
        CXIdxEntityLang_C: 1,
        CXIdxEntityLang_ObjC: 2,
        CXIdxEntityLang_CXX: 3,
        CXIdxObjCContainer_ForwardRef: 0,
        CXIdxObjCContainer_Interface: 1,
        CXIdxObjCContainer_Implementation: 2,
        CXIdxAttr_Unexposed: 0,
        CXIdxAttr_IBAction: 1,
        CXIdxAttr_IBOutlet: 2,
        CXIdxAttr_IBOutletCollection: 3
    }
};
var CXString = exports.CXString = Struct({
    data: voidPtr,
    private_flags: ref.types.uint32,
});
var CXStringPtr = exports.CXStringPtr = ref.refType(CXString);
var CXIndex = exports.CXIndex = voidPtr;
var CXIndexPtr = exports.CXIndexPtr = ref.refType(CXIndex);
var CXFile = exports.CXFile = voidPtr;
var CXFilePtr = exports.CXFilePtr = ref.refType(CXFile);
var CXTranslationUnit = exports.CXTranslationUnit = voidPtr;
var CXTranslationUnitPtr = exports.CXTranslationUnitPtr = ref.refType(CXTranslationUnit);
var CXSourceLocation = exports.CXSourceLocation = Struct({
    ptr_data: ArrayType(voidPtr, 2),
    int_data: ref.types.uint32,
});
var CXSourceLocationPtr = exports.CXSourceLocationPtr = ref.refType(CXSourceLocation);
var CXSourceRange = exports.CXSourceRange = Struct({
    ptr_data: ArrayType(voidPtr, 2),
    begin_int_data: ref.types.uint32,
    end_int_data: ref.types.uint32,
});
var CXSourceRangePtr = exports.CXSourceRangePtr = ref.refType(CXSourceRange);
var CXDiagnosticSet = exports.CXDiagnosticSet = voidPtr;
var CXDiagnosticSetPtr = exports.CXDiagnosticSetPtr = ref.refType(CXDiagnosticSet);
var CXDiagnostic = exports.CXDiagnostic = voidPtr;
var CXDiagnosticPtr = exports.CXDiagnosticPtr = ref.refType(CXDiagnostic);
var CXTUResourceUsageEntry = exports.CXTUResourceUsageEntry = Struct({
    kind: ref.types.uint32,
    amount: ref.types.ulong,
});
var CXTUResourceUsageEntryPtr = exports.CXTUResourceUsageEntryPtr = ref.refType(CXTUResourceUsageEntry);
var CXTUResourceUsage = exports.CXTUResourceUsage = Struct({
    data: voidPtr,
    numEntries: ref.types.uint32,
    entries: CXTUResourceUsageEntryPtr,
});
var CXTUResourceUsagePtr = exports.CXTUResourceUsagePtr = ref.refType(CXTUResourceUsage);
var CXCursor = exports.CXCursor = Struct({
    kind: ref.types.uint32,
    xdata: ref.types.int32,
    data: ArrayType(voidPtr, 3),
});
var CXCursorPtr = exports.CXCursorPtr = ref.refType(CXCursor);
var CXVersion = exports.CXVersion = Struct({
    Major: ref.types.int32,
    Minor: ref.types.int32,
    Subminor: ref.types.int32,
});
var CXVersionPtr = exports.CXVersionPtr = ref.refType(CXVersion);
var CXPlatformAvailability = exports.CXPlatformAvailability = Struct({
    Platform: CXString,
    Introduced: CXVersion,
    Deprecated: CXVersion,
    Obsoleted: CXVersion,
    Unavailable: ref.types.int32,
    Message: CXString,
});
var CXPlatformAvailabilityPtr = exports.CXPlatformAvailabilityPtr = ref.refType(CXPlatformAvailability);
var CXCursorSet = exports.CXCursorSet = voidPtr;
var CXCursorSetPtr = exports.CXCursorSetPtr = ref.refType(CXCursorSet);
var CXType = exports.CXType = Struct({
    kind: ref.types.uint32,
    data: ArrayType(voidPtr, 2),
});
var CXTypePtr = exports.CXTypePtr = ref.refType(CXType);
var CXCursorVisitor = exports.CXCursorVisitor = FFI.Function(ref.types.uint32, [CXCursor, CXCursor, ref.types.uint32]);
var CXClientData = exports.CXClientData = ref.types.uint32;
var CXClientDataPtr = exports.CXClientDataPtr = ref.refType(CXClientData);
var CXCursorVisitorBlock = exports.CXCursorVisitorBlock = Struct({
    CXChildVisitResult: ref.types.uint32,
    cursor: CXCursor,
    parent: CXCursor,
});
var CXCursorVisitorBlockPtr = exports.CXCursorVisitorBlockPtr = ref.refType(CXCursorVisitorBlock);
var CXComment = exports.CXComment = Struct({
    ASTNode: voidPtr,
    TranslationUnit: CXTranslationUnit,
});
var CXCommentPtr = exports.CXCommentPtr = ref.refType(CXComment);
var CXModule = exports.CXModule = voidPtr;
var CXModulePtr = exports.CXModulePtr = ref.refType(CXModule);
var CXToken = exports.CXToken = Struct({
    int_data: ArrayType(ref.types.uint32, 4),
    ptr_data: voidPtr,
});
var CXTokenPtr = exports.CXTokenPtr = ref.refType(CXToken);
var CXCompletionString = exports.CXCompletionString = voidPtr;
var CXCompletionStringPtr = exports.CXCompletionStringPtr = ref.refType(CXCompletionString);
var CXCompletionResult = exports.CXCompletionResult = Struct({
    CursorKind: ref.types.uint32,
    CompletionString: CXCompletionString,
});
var CXCompletionResultPtr = exports.CXCompletionResultPtr = ref.refType(CXCompletionResult);
var CXCodeCompleteResults = exports.CXCodeCompleteResults = Struct({
    Results: CXCompletionResultPtr,
    NumResults: ref.types.uint32,
});
var CXCodeCompleteResultsPtr = exports.CXCodeCompleteResultsPtr = ref.refType(CXCodeCompleteResults);
var CXInclusionVisitor = exports.CXInclusionVisitor = FFI.Function(ref.types.void, [voidPtr, voidPtr, ref.types.uint32, voidPtr]);
var CXRemapping = exports.CXRemapping = voidPtr;
var CXRemappingPtr = exports.CXRemappingPtr = ref.refType(CXRemapping);
var CXCursorAndRangeVisitor = exports.CXCursorAndRangeVisitor = Struct({
    context: voidPtr,
    visit: voidPtr,
});
var CXCursorAndRangeVisitorPtr = exports.CXCursorAndRangeVisitorPtr = ref.refType(CXCursorAndRangeVisitor);
var CXIdxEntityInfo = exports.CXIdxEntityInfo = Struct({
    kind: ref.types.uint32,
    templateKind: ref.types.uint32,
    lang: ref.types.uint32,
    name: ref.types.CString,
    USR: ref.types.CString,
    cursor: CXCursor,
    attributes: voidPtr,
    numAttributes: ref.types.uint32,
});
var CXIdxEntityInfoPtr = exports.CXIdxEntityInfoPtr = ref.refType(CXIdxEntityInfo);
var CXIdxLoc = exports.CXIdxLoc = Struct({
    ptr_data: ArrayType(voidPtr, 2),
    int_data: ref.types.uint32,
});
var CXIdxLocPtr = exports.CXIdxLocPtr = ref.refType(CXIdxLoc);
var CXIdxContainerInfo = exports.CXIdxContainerInfo = Struct({
    cursor: CXCursor,
});
var CXIdxContainerInfoPtr = exports.CXIdxContainerInfoPtr = ref.refType(CXIdxContainerInfo);
var CXIdxDeclInfo = exports.CXIdxDeclInfo = Struct({
    entityInfo: CXIdxEntityInfoPtr,
    cursor: CXCursor,
    loc: CXIdxLoc,
    semanticContainer: CXIdxContainerInfoPtr,
    lexicalContainer: CXIdxContainerInfoPtr,
    isRedeclaration: ref.types.int32,
    isDefinition: ref.types.int32,
    isContainer: ref.types.int32,
    declAsContainer: CXIdxContainerInfoPtr,
    isImplicit: ref.types.int32,
    attributes: voidPtr,
    numAttributes: ref.types.uint32,
});
var CXIdxDeclInfoPtr = exports.CXIdxDeclInfoPtr = ref.refType(CXIdxDeclInfo);
var CXIdxObjCContainerDeclInfo = exports.CXIdxObjCContainerDeclInfo = Struct({
    declInfo: CXIdxDeclInfoPtr,
    kind: ref.types.uint32,
});
var CXIdxObjCContainerDeclInfoPtr = exports.CXIdxObjCContainerDeclInfoPtr = ref.refType(CXIdxObjCContainerDeclInfo);
var CXIdxBaseClassInfo = exports.CXIdxBaseClassInfo = Struct({
    base: CXIdxEntityInfoPtr,
    cursor: CXCursor,
    loc: CXIdxLoc,
});
var CXIdxBaseClassInfoPtr = exports.CXIdxBaseClassInfoPtr = ref.refType(CXIdxBaseClassInfo);
var CXIdxObjCProtocolRefListInfo = exports.CXIdxObjCProtocolRefListInfo = Struct({
    protocols: voidPtr,
    numProtocols: ref.types.uint32,
});
var CXIdxObjCProtocolRefListInfoPtr = exports.CXIdxObjCProtocolRefListInfoPtr = ref.refType(CXIdxObjCProtocolRefListInfo);
var CXIdxObjCInterfaceDeclInfo = exports.CXIdxObjCInterfaceDeclInfo = Struct({
    containerInfo: CXIdxObjCContainerDeclInfoPtr,
    superInfo: CXIdxBaseClassInfoPtr,
    protocols: CXIdxObjCProtocolRefListInfoPtr,
});
var CXIdxObjCInterfaceDeclInfoPtr = exports.CXIdxObjCInterfaceDeclInfoPtr = ref.refType(CXIdxObjCInterfaceDeclInfo);
var CXIdxObjCCategoryDeclInfo = exports.CXIdxObjCCategoryDeclInfo = Struct({
    containerInfo: CXIdxObjCContainerDeclInfoPtr,
    objcClass: CXIdxEntityInfoPtr,
    classCursor: CXCursor,
    classLoc: CXIdxLoc,
    protocols: CXIdxObjCProtocolRefListInfoPtr,
});
var CXIdxObjCCategoryDeclInfoPtr = exports.CXIdxObjCCategoryDeclInfoPtr = ref.refType(CXIdxObjCCategoryDeclInfo);
var CXIdxObjCPropertyDeclInfo = exports.CXIdxObjCPropertyDeclInfo = Struct({
    declInfo: CXIdxDeclInfoPtr,
    getter: CXIdxEntityInfoPtr,
    setter: CXIdxEntityInfoPtr,
});
var CXIdxObjCPropertyDeclInfoPtr = exports.CXIdxObjCPropertyDeclInfoPtr = ref.refType(CXIdxObjCPropertyDeclInfo);
var CXIdxAttrInfo = exports.CXIdxAttrInfo = Struct({
    kind: ref.types.uint32,
    cursor: CXCursor,
    loc: CXIdxLoc,
});
var CXIdxAttrInfoPtr = exports.CXIdxAttrInfoPtr = ref.refType(CXIdxAttrInfo);
var CXIdxIBOutletCollectionAttrInfo = exports.CXIdxIBOutletCollectionAttrInfo = Struct({
    attrInfo: CXIdxAttrInfoPtr,
    objcClass: CXIdxEntityInfoPtr,
    classCursor: CXCursor,
    classLoc: CXIdxLoc,
});
var CXIdxIBOutletCollectionAttrInfoPtr = exports.CXIdxIBOutletCollectionAttrInfoPtr = ref.refType(CXIdxIBOutletCollectionAttrInfo);
var CXIdxCXXClassDeclInfo = exports.CXIdxCXXClassDeclInfo = Struct({
    declInfo: CXIdxDeclInfoPtr,
    bases: voidPtr,
    numBases: ref.types.uint32,
});
var CXIdxCXXClassDeclInfoPtr = exports.CXIdxCXXClassDeclInfoPtr = ref.refType(CXIdxCXXClassDeclInfo);
var CXIdxClientContainer = exports.CXIdxClientContainer = voidPtr;
var CXIdxClientContainerPtr = exports.CXIdxClientContainerPtr = ref.refType(CXIdxClientContainer);
var CXIdxClientEntity = exports.CXIdxClientEntity = voidPtr;
var CXIdxClientEntityPtr = exports.CXIdxClientEntityPtr = ref.refType(CXIdxClientEntity);
var CXIndexAction = exports.CXIndexAction = voidPtr;
var CXIndexActionPtr = exports.CXIndexActionPtr = ref.refType(CXIndexAction);
var IndexerCallbacks = exports.IndexerCallbacks = Struct({
    abortQuery: voidPtr,
    diagnostic: voidPtr,
    enteredMainFile: voidPtr,
    ppIncludedFile: voidPtr,
    importedASTFile: voidPtr,
    startedTranslationUnit: voidPtr,
    indexDeclaration: voidPtr,
    indexEntityReference: voidPtr,
});
var IndexerCallbacksPtr = exports.IndexerCallbacksPtr = ref.refType(IndexerCallbacks);
var CXIdxClientFile = exports.CXIdxClientFile = voidPtr;
var CXIdxClientFilePtr = exports.CXIdxClientFilePtr = ref.refType(CXIdxClientFile);
exports.libclang = new FFI.Library('libclang', {
    clang_getCString: [ref.types.CString, [CXString]],
    clang_disposeString: [ref.types.void, [CXString]],
    clang_createIndex: [CXIndex, [ref.types.int32, ref.types.int32]],
    clang_disposeIndex: [ref.types.void, [CXIndex]],
    clang_CXIndex_setGlobalOptions: [ref.types.void, [CXIndex, ref.types.uint32]],
    clang_CXIndex_getGlobalOptions: [ref.types.uint32, [CXIndex]],
    clang_getFileName: [CXString, [CXFile]],
    clang_getFileTime: [ref.types.long, [CXFile]],
    clang_isFileMultipleIncludeGuarded: [ref.types.uint32, [CXTranslationUnit, CXFile]],
    clang_getFile: [CXFile, [CXTranslationUnit, ref.types.CString]],
    clang_getNullLocation: [CXSourceLocation, []],
    clang_equalLocations: [ref.types.uint32, [CXSourceLocation, CXSourceLocation]],
    clang_getLocation: [CXSourceLocation, [CXTranslationUnit, CXFile, ref.types.uint32, ref.types.uint32]],
    clang_getLocationForOffset: [CXSourceLocation, [CXTranslationUnit, CXFile, ref.types.uint32]],
    clang_getNullRange: [CXSourceRange, []],
    clang_getRange: [CXSourceRange, [CXSourceLocation, CXSourceLocation]],
    clang_equalRanges: [ref.types.uint32, [CXSourceRange, CXSourceRange]],
    clang_Range_isNull: [ref.types.int32, [CXSourceRange]],
    clang_getExpansionLocation: [ref.types.void, [CXSourceLocation, CXFilePtr, voidPtr, voidPtr, voidPtr]],
    clang_getPresumedLocation: [ref.types.void, [CXSourceLocation, CXStringPtr, voidPtr, voidPtr]],
    clang_getInstantiationLocation: [ref.types.void, [CXSourceLocation, CXFilePtr, voidPtr, voidPtr, voidPtr]],
    clang_getSpellingLocation: [ref.types.void, [CXSourceLocation, CXFilePtr, voidPtr, voidPtr, voidPtr]],
    clang_getRangeStart: [CXSourceLocation, [CXSourceRange]],
    clang_getRangeEnd: [CXSourceLocation, [CXSourceRange]],
    clang_getNumDiagnosticsInSet: [ref.types.uint32, [CXDiagnosticSet]],
    clang_getDiagnosticInSet: [CXDiagnostic, [CXDiagnosticSet, ref.types.uint32]],
    clang_loadDiagnostics: [CXDiagnosticSet, [ref.types.CString, voidPtr, CXStringPtr]],
    clang_disposeDiagnosticSet: [ref.types.void, [CXDiagnosticSet]],
    clang_getChildDiagnostics: [CXDiagnosticSet, [CXDiagnostic]],
    clang_getNumDiagnostics: [ref.types.uint32, [CXTranslationUnit]],
    clang_getDiagnostic: [CXDiagnostic, [CXTranslationUnit, ref.types.uint32]],
    clang_getDiagnosticSetFromTU: [CXDiagnosticSet, [CXTranslationUnit]],
    clang_disposeDiagnostic: [ref.types.void, [CXDiagnostic]],
    clang_formatDiagnostic: [CXString, [CXDiagnostic, ref.types.uint32]],
    clang_defaultDiagnosticDisplayOptions: [ref.types.uint32, []],
    clang_getDiagnosticSeverity: [ref.types.uint32, [CXDiagnostic]],
    clang_getDiagnosticLocation: [CXSourceLocation, [CXDiagnostic]],
    clang_getDiagnosticSpelling: [CXString, [CXDiagnostic]],
    clang_getDiagnosticOption: [CXString, [CXDiagnostic, CXStringPtr]],
    clang_getDiagnosticCategory: [ref.types.uint32, [CXDiagnostic]],
    clang_getDiagnosticCategoryName: [CXString, [ref.types.uint32]],
    clang_getDiagnosticCategoryText: [CXString, [CXDiagnostic]],
    clang_getDiagnosticNumRanges: [ref.types.uint32, [CXDiagnostic]],
    clang_getDiagnosticRange: [CXSourceRange, [CXDiagnostic, ref.types.uint32]],
    clang_getDiagnosticNumFixIts: [ref.types.uint32, [CXDiagnostic]],
    clang_getDiagnosticFixIt: [CXString, [CXDiagnostic, ref.types.uint32, CXSourceRangePtr]],
    clang_getTranslationUnitSpelling: [CXString, [CXTranslationUnit]],
    clang_createTranslationUnitFromSourceFile: [CXTranslationUnit, [CXIndex, ref.types.CString, ref.types.int32, voidPtr, ref.types.uint32, voidPtr]],
    clang_createTranslationUnit: [CXTranslationUnit, [CXIndex, ref.types.CString]],
    clang_defaultEditingTranslationUnitOptions: [ref.types.uint32, []],
    clang_parseTranslationUnit: [CXTranslationUnit, [CXIndex, ref.types.CString, voidPtr, ref.types.int32, voidPtr, ref.types.uint32, ref.types.uint32]],
    clang_defaultSaveOptions: [ref.types.uint32, [CXTranslationUnit]],
    clang_saveTranslationUnit: [ref.types.int32, [CXTranslationUnit, ref.types.CString, ref.types.uint32]],
    clang_disposeTranslationUnit: [ref.types.void, [CXTranslationUnit]],
    clang_defaultReparseOptions: [ref.types.uint32, [CXTranslationUnit]],
    clang_reparseTranslationUnit: [ref.types.int32, [CXTranslationUnit, ref.types.uint32, voidPtr, ref.types.uint32]],
    clang_getTUResourceUsageName: [ref.types.CString, [ref.types.uint32]],
    clang_getCXTUResourceUsage: [CXTUResourceUsage, [CXTranslationUnit]],
    clang_disposeCXTUResourceUsage: [ref.types.void, [CXTUResourceUsage]],
    clang_getNullCursor: [CXCursor, []],
    clang_getTranslationUnitCursor: [CXCursor, [CXTranslationUnit]],
    clang_equalCursors: [ref.types.uint32, [CXCursor, CXCursor]],
    clang_Cursor_isNull: [ref.types.int32, [CXCursor]],
    clang_hashCursor: [ref.types.uint32, [CXCursor]],
    clang_getCursorKind: [ref.types.uint32, [CXCursor]],
    clang_isDeclaration: [ref.types.uint32, [ref.types.uint32]],
    clang_isReference: [ref.types.uint32, [ref.types.uint32]],
    clang_isExpression: [ref.types.uint32, [ref.types.uint32]],
    clang_isStatement: [ref.types.uint32, [ref.types.uint32]],
    clang_isAttribute: [ref.types.uint32, [ref.types.uint32]],
    clang_isInvalid: [ref.types.uint32, [ref.types.uint32]],
    clang_isTranslationUnit: [ref.types.uint32, [ref.types.uint32]],
    clang_isPreprocessing: [ref.types.uint32, [ref.types.uint32]],
    clang_isUnexposed: [ref.types.uint32, [ref.types.uint32]],
    clang_getCursorLinkage: [ref.types.uint32, [CXCursor]],
    clang_getCursorAvailability: [ref.types.uint32, [CXCursor]],
    clang_getCursorPlatformAvailability: [ref.types.int32, [CXCursor, voidPtr, CXStringPtr, voidPtr, CXStringPtr, CXPlatformAvailabilityPtr, ref.types.int32]],
    clang_disposeCXPlatformAvailability: [ref.types.void, [CXPlatformAvailabilityPtr]],
    clang_getCursorLanguage: [ref.types.uint32, [CXCursor]],
    clang_Cursor_getTranslationUnit: [CXTranslationUnit, [CXCursor]],
    clang_createCXCursorSet: [CXCursorSet, []],
    clang_disposeCXCursorSet: [ref.types.void, [CXCursorSet]],
    clang_CXCursorSet_contains: [ref.types.uint32, [CXCursorSet, CXCursor]],
    clang_CXCursorSet_insert: [ref.types.uint32, [CXCursorSet, CXCursor]],
    clang_getCursorSemanticParent: [CXCursor, [CXCursor]],
    clang_getCursorLexicalParent: [CXCursor, [CXCursor]],
    clang_getOverriddenCursors: [ref.types.void, [CXCursor, voidPtr, voidPtr]],
    clang_disposeOverriddenCursors: [ref.types.void, [CXCursorPtr]],
    clang_getIncludedFile: [CXFile, [CXCursor]],
    clang_getCursor: [CXCursor, [CXTranslationUnit, CXSourceLocation]],
    clang_getCursorLocation: [CXSourceLocation, [CXCursor]],
    clang_getCursorExtent: [CXSourceRange, [CXCursor]],
    clang_getCursorType: [CXType, [CXCursor]],
    clang_getTypedefDeclUnderlyingType: [CXType, [CXCursor]],
    clang_getEnumDeclIntegerType: [CXType, [CXCursor]],
    clang_getEnumConstantDeclValue: [ref.types.longlong, [CXCursor]],
    clang_getEnumConstantDeclUnsignedValue: [ref.types.ulonglong, [CXCursor]],
    clang_Cursor_getNumArguments: [ref.types.int32, [CXCursor]],
    clang_Cursor_getArgument: [CXCursor, [CXCursor, ref.types.uint32]],
    clang_equalTypes: [ref.types.uint32, [CXType, CXType]],
    clang_getCanonicalType: [CXType, [CXType]],
    clang_isConstQualifiedType: [ref.types.uint32, [CXType]],
    clang_isVolatileQualifiedType: [ref.types.uint32, [CXType]],
    clang_isRestrictQualifiedType: [ref.types.uint32, [CXType]],
    clang_getPointeeType: [CXType, [CXType]],
    clang_getTypeDeclaration: [CXCursor, [CXType]],
    clang_getDeclObjCTypeEncoding: [CXString, [CXCursor]],
    clang_getTypeSpelling: [CXString, [CXType]],
    clang_getFunctionTypeCallingConv: [ref.types.uint32, [CXType]],
    clang_getResultType: [CXType, [CXType]],
    clang_getNumArgTypes: [ref.types.int32, [CXType]],
    clang_getArgType: [CXType, [CXType, ref.types.uint32]],
    clang_isFunctionTypeVariadic: [ref.types.uint32, [CXType]],
    clang_getCursorResultType: [CXType, [CXCursor]],
    clang_isPODType: [ref.types.uint32, [CXType]],
    clang_getElementType: [CXType, [CXType]],
    clang_getNumElements: [ref.types.longlong, [CXType]],
    clang_getArrayElementType: [CXType, [CXType]],
    clang_getArraySize: [ref.types.longlong, [CXType]],
    clang_isVirtualBase: [ref.types.uint32, [CXCursor]],
    clang_getCXXAccessSpecifier: [ref.types.uint32, [CXCursor]],
    clang_getNumOverloadedDecls: [ref.types.uint32, [CXCursor]],
    clang_getOverloadedDecl: [CXCursor, [CXCursor, ref.types.uint32]],
    clang_getIBOutletCollectionType: [CXType, [CXCursor]],
    clang_visitChildren: [ref.types.uint32, [CXCursor, CXCursorVisitor, CXClientData]],
    //clang_visitChildrenWithBlock: [ref.types.uint32, [CXCursor, CXCursorVisitorBlock]],
    clang_getCursorUSR: [CXString, [CXCursor]],
    clang_constructUSR_ObjCClass: [CXString, [ref.types.CString]],
    clang_constructUSR_ObjCCategory: [CXString, [ref.types.CString, ref.types.CString]],
    clang_constructUSR_ObjCProtocol: [CXString, [ref.types.CString]],
    clang_constructUSR_ObjCIvar: [CXString, [ref.types.CString, CXString]],
    clang_constructUSR_ObjCMethod: [CXString, [ref.types.CString, ref.types.uint32, CXString]],
    clang_constructUSR_ObjCProperty: [CXString, [ref.types.CString, CXString]],
    clang_getCursorSpelling: [CXString, [CXCursor]],
    clang_Cursor_getSpellingNameRange: [CXSourceRange, [CXCursor, ref.types.uint32, ref.types.uint32]],
    clang_getCursorDisplayName: [CXString, [CXCursor]],
    clang_getCursorReferenced: [CXCursor, [CXCursor]],
    clang_getCursorDefinition: [CXCursor, [CXCursor]],
    clang_isCursorDefinition: [ref.types.uint32, [CXCursor]],
    clang_getCanonicalCursor: [CXCursor, [CXCursor]],
    clang_Cursor_getObjCSelectorIndex: [ref.types.int32, [CXCursor]],
    clang_Cursor_isDynamicCall: [ref.types.int32, [CXCursor]],
    //clang_Cursor_getReceiverType: [CXType, [CXCursor]],
    clang_Cursor_getCommentRange: [CXSourceRange, [CXCursor]],
    clang_Cursor_getRawCommentText: [CXString, [CXCursor]],
    clang_Cursor_getBriefCommentText: [CXString, [CXCursor]],
    clang_Cursor_getParsedComment: [CXComment, [CXCursor]],
    //clang_Cursor_getModule: [CXModule, [CXCursor]],
    //clang_Module_getParent: [CXModule, [CXModule]],
    //clang_Module_getName: [CXString, [CXModule]],
    //clang_Module_getFullName: [CXString, [CXModule]],
    //clang_Module_getNumTopLevelHeaders: [ref.types.uint32, [CXModule]],
    //clang_Module_getTopLevelHeader: [CXFile, [CXModule, ref.types.uint32]],
    clang_Comment_getKind: [ref.types.uint32, [CXComment]],
    clang_Comment_getNumChildren: [ref.types.uint32, [CXComment]],
    clang_Comment_getChild: [CXComment, [CXComment, ref.types.uint32]],
    clang_Comment_isWhitespace: [ref.types.uint32, [CXComment]],
    clang_InlineContentComment_hasTrailingNewline: [ref.types.uint32, [CXComment]],
    clang_TextComment_getText: [CXString, [CXComment]],
    clang_InlineCommandComment_getCommandName: [CXString, [CXComment]],
    clang_InlineCommandComment_getRenderKind: [ref.types.uint32, [CXComment]],
    clang_InlineCommandComment_getNumArgs: [ref.types.uint32, [CXComment]],
    clang_InlineCommandComment_getArgText: [CXString, [CXComment, ref.types.uint32]],
    clang_HTMLTagComment_getTagName: [CXString, [CXComment]],
    clang_HTMLStartTagComment_isSelfClosing: [ref.types.uint32, [CXComment]],
    clang_HTMLStartTag_getNumAttrs: [ref.types.uint32, [CXComment]],
    clang_HTMLStartTag_getAttrName: [CXString, [CXComment, ref.types.uint32]],
    clang_HTMLStartTag_getAttrValue: [CXString, [CXComment, ref.types.uint32]],
    clang_BlockCommandComment_getCommandName: [CXString, [CXComment]],
    clang_BlockCommandComment_getNumArgs: [ref.types.uint32, [CXComment]],
    clang_BlockCommandComment_getArgText: [CXString, [CXComment, ref.types.uint32]],
    clang_BlockCommandComment_getParagraph: [CXComment, [CXComment]],
    clang_ParamCommandComment_getParamName: [CXString, [CXComment]],
    clang_ParamCommandComment_isParamIndexValid: [ref.types.uint32, [CXComment]],
    clang_ParamCommandComment_getParamIndex: [ref.types.uint32, [CXComment]],
    clang_ParamCommandComment_isDirectionExplicit: [ref.types.uint32, [CXComment]],
    clang_ParamCommandComment_getDirection: [ref.types.uint32, [CXComment]],
    clang_TParamCommandComment_getParamName: [CXString, [CXComment]],
    clang_TParamCommandComment_isParamPositionValid: [ref.types.uint32, [CXComment]],
    clang_TParamCommandComment_getDepth: [ref.types.uint32, [CXComment]],
    clang_TParamCommandComment_getIndex: [ref.types.uint32, [CXComment, ref.types.uint32]],
    clang_VerbatimBlockLineComment_getText: [CXString, [CXComment]],
    clang_VerbatimLineComment_getText: [CXString, [CXComment]],
    clang_HTMLTagComment_getAsString: [CXString, [CXComment]],
    clang_FullComment_getAsHTML: [CXString, [CXComment]],
    clang_FullComment_getAsXML: [CXString, [CXComment]],
    clang_CXXMethod_isStatic: [ref.types.uint32, [CXCursor]],
    clang_CXXMethod_isVirtual: [ref.types.uint32, [CXCursor]],
    clang_getTemplateCursorKind: [ref.types.uint32, [CXCursor]],
    clang_getSpecializedCursorTemplate: [CXCursor, [CXCursor]],
    clang_getCursorReferenceNameRange: [CXSourceRange, [CXCursor, ref.types.uint32, ref.types.uint32]],
    clang_getTokenKind: [ref.types.uint32, [CXToken]],
    clang_getTokenSpelling: [CXString, [CXTranslationUnit, CXToken]],
    clang_getTokenLocation: [CXSourceLocation, [CXTranslationUnit, CXToken]],
    clang_getTokenExtent: [CXSourceRange, [CXTranslationUnit, CXToken]],
    clang_tokenize: [ref.types.void, [CXTranslationUnit, CXSourceRange, voidPtr, voidPtr]],
    clang_annotateTokens: [ref.types.void, [CXTranslationUnit, CXTokenPtr, ref.types.uint32, CXCursorPtr]],
    clang_disposeTokens: [ref.types.void, [CXTranslationUnit, CXTokenPtr, ref.types.uint32]],
    clang_getCursorKindSpelling: [CXString, [ref.types.uint32]],
    clang_getDefinitionSpellingAndExtent: [ref.types.void, [CXCursor, voidPtr, voidPtr, voidPtr, voidPtr, voidPtr, voidPtr]],
    clang_enableStackTraces: [ref.types.void, []],
    clang_executeOnThread: [ref.types.void, [voidPtr, voidPtr, ref.types.uint32]],
    clang_getCompletionChunkKind: [ref.types.uint32, [CXCompletionString, ref.types.uint32]],
    clang_getCompletionChunkText: [CXString, [CXCompletionString, ref.types.uint32]],
    clang_getCompletionChunkCompletionString: [CXCompletionString, [CXCompletionString, ref.types.uint32]],
    clang_getNumCompletionChunks: [ref.types.uint32, [CXCompletionString]],
    clang_getCompletionPriority: [ref.types.uint32, [CXCompletionString]],
    clang_getCompletionAvailability: [ref.types.uint32, [CXCompletionString]],
    clang_getCompletionNumAnnotations: [ref.types.uint32, [CXCompletionString]],
    clang_getCompletionAnnotation: [CXString, [CXCompletionString, ref.types.uint32]],
    clang_getCompletionParent: [CXString, [CXCompletionString, voidPtr]],
    clang_getCompletionBriefComment: [CXString, [CXCompletionString]],
    clang_getCursorCompletionString: [CXCompletionString, [CXCursor]],
    clang_defaultCodeCompleteOptions: [ref.types.uint32, []],
    clang_codeCompleteAt: [CXCodeCompleteResultsPtr, [CXTranslationUnit, ref.types.CString, ref.types.uint32, ref.types.uint32, voidPtr, ref.types.uint32, ref.types.uint32]],
    clang_sortCodeCompletionResults: [ref.types.void, [CXCompletionResultPtr, ref.types.uint32]],
    clang_disposeCodeCompleteResults: [ref.types.void, [CXCodeCompleteResultsPtr]],
    clang_codeCompleteGetNumDiagnostics: [ref.types.uint32, [CXCodeCompleteResultsPtr]],
    clang_codeCompleteGetDiagnostic: [CXDiagnostic, [CXCodeCompleteResultsPtr, ref.types.uint32]],
    clang_codeCompleteGetContexts: [ref.types.ulonglong, [CXCodeCompleteResultsPtr]],
    clang_codeCompleteGetContainerKind: [ref.types.uint32, [CXCodeCompleteResultsPtr, voidPtr]],
    clang_codeCompleteGetContainerUSR: [CXString, [CXCodeCompleteResultsPtr]],
    clang_codeCompleteGetObjCSelector: [CXString, [CXCodeCompleteResultsPtr]],
    clang_getClangVersion: [CXString, []],
    clang_toggleCrashRecovery: [ref.types.void, [ref.types.uint32]],
    clang_getInclusions: [ref.types.void, [CXTranslationUnit, CXInclusionVisitor, CXClientData]],
    clang_getRemappings: [CXRemapping, [ref.types.CString]],
    clang_getRemappingsFromFileList: [CXRemapping, [voidPtr, ref.types.uint32]],
    clang_remap_getNumFiles: [ref.types.uint32, [CXRemapping]],
    clang_remap_getFilenames: [ref.types.void, [CXRemapping, ref.types.uint32, CXStringPtr, CXStringPtr]],
    clang_remap_dispose: [ref.types.void, [CXRemapping]],
    clang_findReferencesInFile: [ref.types.void, [CXCursor, CXFile, CXCursorAndRangeVisitor]],
    clang_index_isEntityObjCContainerKind: [ref.types.int32, [ref.types.uint32]],
    clang_index_getObjCContainerDeclInfo: [CXIdxObjCContainerDeclInfoPtr, [CXIdxDeclInfoPtr]],
    clang_index_getObjCInterfaceDeclInfo: [CXIdxObjCInterfaceDeclInfoPtr, [CXIdxDeclInfoPtr]],
    clang_index_getObjCCategoryDeclInfo: [CXIdxObjCCategoryDeclInfoPtr, [CXIdxDeclInfoPtr]],
    clang_index_getObjCProtocolRefListInfo: [CXIdxObjCProtocolRefListInfoPtr, [CXIdxDeclInfoPtr]],
    clang_index_getObjCPropertyDeclInfo: [CXIdxObjCPropertyDeclInfoPtr, [CXIdxDeclInfoPtr]],
    clang_index_getIBOutletCollectionAttrInfo: [CXIdxIBOutletCollectionAttrInfoPtr, [CXIdxAttrInfoPtr]],
    clang_index_getCXXClassDeclInfo: [CXIdxCXXClassDeclInfoPtr, [CXIdxDeclInfoPtr]],
    clang_index_getClientContainer: [CXIdxClientContainer, [CXIdxContainerInfoPtr]],
    clang_index_setClientContainer: [ref.types.void, [CXIdxContainerInfoPtr, CXIdxClientContainer]],
    clang_index_getClientEntity: [CXIdxClientEntity, [CXIdxEntityInfoPtr]],
    clang_index_setClientEntity: [ref.types.void, [CXIdxEntityInfoPtr, CXIdxClientEntity]],
    clang_IndexAction_create: [CXIndexAction, [CXIndex]],
    clang_IndexAction_dispose: [ref.types.void, [CXIndexAction]],
    clang_indexSourceFile: [ref.types.int32, [CXIndexAction, CXClientData, IndexerCallbacksPtr, ref.types.uint32, ref.types.uint32, ref.types.CString, voidPtr, ref.types.int32, voidPtr, ref.types.uint32, CXTranslationUnitPtr, ref.types.uint32]],
    clang_indexTranslationUnit: [ref.types.int32, [CXIndexAction, CXClientData, IndexerCallbacksPtr, ref.types.uint32, ref.types.uint32, CXTranslationUnit]],
    clang_indexLoc_getFileLocation: [ref.types.void, [CXIdxLoc, CXIdxClientFile, CXFilePtr, voidPtr, voidPtr, voidPtr]],
    clang_indexLoc_getCXSourceLocation: [CXSourceLocation, [CXIdxLoc]],
});
