"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
let fs = require('fs-extra');
let path = require('path');
let tmp = require('tmp');
let crypto = require('crypto');
/**
 * Implements a cache on the filesystem.
 *
 * @author Andrew Tch <andrew@noop.lv>
 */
class TwingCacheFilesystem {
    /**
     * @param directory {string} The root cache directory
     * @param options {number} A set of options
     */
    constructor(directory, options = 0) {
        this.directory = directory;
        this.options = options;
        this.TwingCacheInterfaceImpl = this;
    }
    generateKey(name, className) {
        let hash = crypto.createHash('sha256').update(className).digest('hex');
        return path.join(this.directory, hash[0] + hash[1], hash + '.js');
    }
    load(key) {
        let modulePath = path.resolve(key);
        if (fs.pathExistsSync(modulePath)) {
            let cache = require.cache;
            delete cache[modulePath];
            return require(modulePath);
        }
        return null;
    }
    write(key, content) {
        let dir = path.dirname(key);
        fs.ensureDirSync(dir);
        let tmpFile = tmp.tmpNameSync({
            dir: dir,
            postfix: path.extname(key, '.js')
        });
        try {
            fs.writeFileSync(tmpFile, content);
            fs.renameSync(tmpFile, key);
            return;
        }
        catch (e) {
            throw new Error(`Failed to write cache file "${key}".`);
        }
    }
    getTimestamp(key) {
        if (!fs.pathExistsSync(key)) {
            return 0;
        }
        let stat = fs.statSync(key);
        return stat.mtimeMs;
    }
}
exports.TwingCacheFilesystem = TwingCacheFilesystem;
