import { TwingTokenParserInterface } from "./token-parser-interface";
import { TwingFilter } from "./filter";
import { TwingFunction } from "./function";
import { TwingNodeVisitorInterface } from "./node-visitor-interface";
import { TwingExtensionInterface } from "./extension-interface";
import { TwingTest } from "./test";
import { TwingEnvironment } from "./environment";
import { TwingOperator } from "./extension";
export declare class TwingExtensionSet {
    private extensions;
    private initialized;
    private runtimeInitialized;
    private staging;
    private parsers;
    private visitors;
    private filters;
    private tests;
    private functions;
    private unaryOperators;
    private binaryOperators;
    private globals;
    private functionCallbacks;
    private filterCallbacks;
    constructor();
    /**
     * Initializes the runtime environment.
     */
    initRuntime(env: TwingEnvironment): void;
    hasExtension(name: string): boolean;
    getExtension(name: string): TwingExtensionInterface;
    /**
     * Registers an array of extensions.
     *
     * @param {Array<TwingExtensionInterface>} extensions An array of extensions
     */
    setExtensions(extensions: Array<TwingExtensionInterface>): void;
    /**
     * Returns all registered extensions.
     *
     * @return Map<string, TwingExtensionInterface> A hash of extensions
     */
    getExtensions(): Map<string, TwingExtensionInterface>;
    getSignature(): string;
    isInitialized(): boolean;
    getNodeVisitors(): TwingNodeVisitorInterface[];
    getTokenParsers(): TwingTokenParserInterface[];
    getGlobals(): Map<any, any>;
    /**
     * Registers an extension.
     *
     * @param {TwingExtensionInterface} extension A TwingExtensionInterface instance
     * @param {String} name An optional name the extension will be registered as
     */
    addExtension(extension: TwingExtensionInterface, name?: string): void;
    addTokenParser(parser: TwingTokenParserInterface): void;
    /**
     * Gets the registered unary Operators.
     *
     * @return Map<string, TwingOperator> A map of unary operator definitions
     */
    getUnaryOperators(): Map<string, TwingOperator>;
    /**
     * Gets the registered binary Operators.
     *
     * @return Map<string, TwingOperator> A map of binary operators
     */
    getBinaryOperators(): Map<string, TwingOperator>;
    addFunction(twingFunction: TwingFunction): void;
    getFunctions(): Map<string, TwingFunction>;
    /**
     * Get a function by name.
     *
     * @param {string} name         function name
     * @returns {TwingFunction}     A TwingFunction instance or null if the function does not exist
     */
    getFunction(name: string): TwingFunction;
    registerUndefinedFunctionCallback(callable: Function): void;
    addFilter(filter: TwingFilter): void;
    getFilters(): Map<string, TwingFilter>;
    /**
     * Get a filter by name.
     *
     * Subclasses may override this method and load filters differently;
     * so no list of filters is available.
     *
     * @param string name The filter name
     *
     * @return Twig_Filter|false A Twig_Filter instance or false if the filter does not exist
     */
    getFilter(name: string): TwingFilter;
    registerUndefinedFilterCallback(callable: Function): void;
    addNodeVisitor(visitor: TwingNodeVisitorInterface): void;
    addTest(test: TwingTest): void;
    /**
     *
     * @returns {Map<string, TwingTest>}
     */
    getTests(): Map<string, TwingTest>;
    /**
     * Gets a test by name.
     *
     * @param {string} name The test name
     * @returns {TwingTest} A TwingTest instance or null if the test does not exist
     */
    getTest(name: string): TwingTest;
    private initExtensions;
    private initExtension;
}
