import { TwingNode } from "./node";
export declare type TwingFunctionOptions = {
    needs_environment?: boolean;
    needs_context?: boolean;
    is_variadic?: boolean;
    is_safe?: Array<string>;
    is_safe_callback?: Function;
    expression_factory?: Function;
    deprecated?: string;
    alternative?: string;
};
export declare class TwingFunction {
    private name;
    private callable;
    private options;
    private arguments;
    /**
     * Creates a template function.
     *
     * @param {string} name Name of this function
     * @param {Function} callable A callable implementing the function. If null, you need to overwrite the "expression_factory" option to customize compilation.
     * @param {TwingFunctionOptions} options Options
     */
    constructor(name: string, callable?: Function, options?: TwingFunctionOptions);
    getName(): string;
    /**
     * Returns the callable to execute for this function.
     *
     * @return callable|null
     */
    getCallable(): Function;
    getExpressionFactory(): Function;
    setArguments(arguments_: Array<any>): void;
    getArguments(): any[];
    needsEnvironment(): boolean;
    needsContext(): boolean;
    getSafe(functionArgs: TwingNode): any;
    isVariadic(): boolean;
    isDeprecated(): boolean;
    getDeprecatedVersion(): string;
    getAlternative(): string;
}
