"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * This one is tricky
 *
 * Equivalence rules is PHP are all over the place.
 * For example an empty array is equivalent to another empty array.
 * Even worse, an empty array is equivalent to false but not to '' but at the same time false and '' are equivalent.
 * This language is a nonsense but we have to conform to its rules.
 *
 * @see http://php.net/manual/en/types.comparisons.php
 * @see https://stackoverflow.com/questions/47969711/php-algorithm-loose-equality-comparison
 */
const luxon_1 = require("luxon");
const compare_to_array_1 = require("./compare-to-array");
const compare_to_string_1 = require("./compare-to-string");
const compare_to_number_1 = require("./compare-to-number");
const compare_to_boolean_1 = require("./compare-to-boolean");
const compare_to_date_time_1 = require("./compare-to-date-time");
const compare_to_null_1 = require("./compare-to-null");
function compare(firstOperand, secondOperand) {
    // array
    if (Array.isArray(firstOperand)) {
        return compare_to_array_1.compareToArray(firstOperand, secondOperand);
    }
    // string
    if (typeof firstOperand === 'string') {
        return compare_to_string_1.compareToString(firstOperand, secondOperand);
    }
    // number
    if (typeof firstOperand === 'number') {
        return compare_to_number_1.compareToNumber(firstOperand, secondOperand);
    }
    // boolean
    if (typeof firstOperand === 'boolean') {
        return compare_to_boolean_1.compareToBoolean(firstOperand, secondOperand);
    }
    // date
    if (firstOperand instanceof luxon_1.DateTime) {
        return compare_to_date_time_1.compareToDateTime(firstOperand, secondOperand);
    }
    // null
    if (firstOperand === null) {
        return compare_to_null_1.compareToNull(secondOperand);
    }
    // fallback to strict comparison
    return firstOperand === secondOperand;
}
exports.compare = compare;
