import { TwingLoaderInterface } from "../loader-interface";
import { TwingSource } from "../source";
/**
 * Loads template from the filesystem.
 *
 * @author Fabien Potencier <fabien@symfony.com>
 * @author Eric MORAND <eric.morand@gmail.com>
 */
export declare class TwingLoaderFilesystem implements TwingLoaderInterface {
    TwingLoaderInterfaceImpl: TwingLoaderInterface;
    /** Identifier of the main namespace. */
    static MAIN_NAMESPACE: string;
    protected paths: Map<string, Array<string>>;
    protected cache: Map<string, string>;
    protected errorCache: Map<string, string>;
    private rootPath;
    /**
     * @param {string | Array<string>} paths A path or a map of paths where to look for templates
     * @param {string} rootPath The root path common to all relative paths (null for process.cwd())
     */
    constructor(paths?: string | Array<string>, rootPath?: string);
    /**
     * Returns the paths to the templates.
     *
     * @param {string} namespace A path namespace
     *
     * @returns Array<string> The array of paths where to look for templates
     */
    getPaths(namespace?: string): Array<string>;
    /**
     * Returns the path namespaces.
     *
     * The main namespace is always defined.
     *
     * @returns Array<string> The array of defined namespaces
     */
    getNamespaces(): Array<string>;
    /**
     * Sets the paths where templates are stored.
     *
     * @param {string|Array<string>} paths A path or an array of paths where to look for templates
     * @param {string} namespace A path namespace
     */
    setPaths(paths: string | Array<string>, namespace?: string): void;
    /**
     * Adds a path where templates are stored.
     *
     * @param {string} path A path where to look for templates
     * @param {string} namespace A path namespace
     *
     * @throws TwingErrorLoader
     */
    addPath(path: string, namespace?: string): void;
    /**
     * Prepends a path where templates are stored.
     *
     * @param {string} path A path where to look for templates
     * @param {string} namespace A path namespace
     *
     * @throws TwingErrorLoader
     */
    prependPath(path: string, namespace?: string): void;
    getSourceContext(name: string): TwingSource;
    getCacheKey(name: string): string;
    exists(name: string): boolean;
    isFresh(name: string, time: number): boolean;
    /**
     * Checks if the template can be found.
     *
     * @param {string} name  The template name
     * @param {boolean} throw_ Whether to throw an exception when an error occurs
     *
     * @returns {string} The template name or null
     */
    protected findTemplate(name: string, throw_?: boolean): string;
    private normalizeName;
    private parseName;
    private validateName;
    private isAbsolutePath;
    resolve(name: string): string;
}
