import { TwingEnvironment } from "./environment";
import { TwingTokenStream } from "./token-stream";
import { TwingNodeBlock } from "./node/block";
import { TwingExpressionParser } from "./expression-parser";
import { TwingNode } from "./node";
import { TwingToken } from "./token";
import { TwingNodeExpression } from "./node/expression";
import { TwingNodeBody } from "./node/body";
import { TwingNodeModule } from "./node/module";
import { TwingNodeMacro } from "./node/macro";
export declare class TwingParser {
    private stack;
    private stream;
    private parent;
    private handlers;
    private visitors;
    private expressionParser;
    private blocks;
    private blockStack;
    private macros;
    private env;
    private importedSymbols;
    private traits;
    private embeddedTemplates;
    private varNameSalt;
    constructor(env: TwingEnvironment);
    getVarName(prefix?: string): string;
    parse(stream: TwingTokenStream, test?: Array<any>, dropNeedle?: boolean): TwingNodeModule;
    getExpressionParser(): TwingExpressionParser;
    getParent(): TwingNode;
    setParent(parent: TwingNode): void;
    subparse(test: Array<any>, dropNeedle?: boolean): TwingNode;
    getBlockStack(): string[];
    peekBlockStack(): string;
    popBlockStack(): void;
    pushBlockStack(name: string): void;
    hasBlock(name: string): boolean;
    getBlock(name: string): TwingNodeBody;
    setBlock(name: string, value: TwingNodeBlock): void;
    addTrait(trait: TwingNode): void;
    hasTraits(): boolean;
    embedTemplate(template: TwingNodeModule): void;
    /**
     * @return {TwingToken}
     */
    getCurrentToken(): TwingToken;
    /**
     *
     * @return {TwingTokenStream}
     */
    getStream(): TwingTokenStream;
    addImportedSymbol(type: string, alias: string, name?: string, node?: TwingNodeExpression): void;
    getImportedSymbol(type: string, alias: string): {
        node: TwingNodeExpression;
        name: string;
    };
    hasMacro(name: string): boolean;
    setMacro(name: string, node: TwingNodeMacro): void;
    isMainScope(): boolean;
    pushLocalScope(): void;
    popLocalScope(): void;
    /**
     *
     * @param node
     * @param nested {boolean}
     * @returns {TwingNode}
     */
    filterBodyNodes(node: TwingNode, nested?: boolean): TwingNode;
}
