/**
 * Exposes a template to userland.
 *
 * @author Eric MORAND <eric.morand@gmail.com>
 */
import { TwingEnvironment } from "./environment";
import { TwingTemplate } from "./template";
import { TwingSource } from "./source";
export declare class TwingTemplateWrapper {
    private env;
    private template;
    /**
     * This method is for internal use only and should never be called
     * directly (use TwingEnvironment::load() instead).
     *
     * @internal
     */
    constructor(env: TwingEnvironment, template: TwingTemplate);
    /**
     * Renders the template.
     *
     * @param {*} context A hash of parameters to pass to the template
     *
     * @returns {string} The rendered template
     */
    render(context?: any): string;
    /**
     * Checks if a block is defined.
     *
     * @param {string} name The block name
     * @param {*} context A hash of parameters to pass to the template
     *
     * @returns {boolean}
     */
    hasBlock(name: string, context?: any): boolean;
    /**
     * Returns defined block names in the template.
     *
     * @param {*} context A hash of parameters to pass to the template
     *
     * @returns {Array<string>} An array of defined template block names
     */
    getBlockNames(context?: any): string[];
    /**
     * Renders a template block.
     *
     * @param {string} name The block name to render
     * @param {*} context A hash of parameters to pass to the template
     *
     * @returns {string} The rendered block
     */
    renderBlock(name: string, context?: any): string;
    /**
     * Displays a template block.
     *
     * @param {string} name The block name to render
     * @param {*} context A hash of parameters to pass to the template
     */
    displayBlock(name: string, context?: any): void;
    /**
     *
     * @returns {TwingSource}
     */
    getSourceContext(): TwingSource;
}
