import { TwingParser } from "./parser";
import { TwingNodeExpression } from "./node/expression";
import { TwingNode } from "./node";
import { TwingEnvironment } from "./environment";
/**
 * Parses expressions.
 *
 * This parser implements a "Precedence climbing" algorithm.
 *
 * @see https://www.engr.mun.ca/~theo/Misc/exp_parsing.htm
 * @see https://en.wikipedia.org/wiki/Operator-precedence_parser
 *
 * @author Fabien Potencier <fabien@symfony.com>
 * @author Eric MORAND <eric.morand@gmail.com>
 *
 * @internal
 */
export declare class TwingExpressionParser {
    static OPERATOR_LEFT: string;
    static OPERATOR_RIGHT: string;
    private parser;
    private env;
    private unaryOperators;
    private binaryOperators;
    constructor(parser: TwingParser, env: TwingEnvironment);
    parseExpression(precedence?: number): TwingNodeExpression;
    getPrimary(): TwingNodeExpression;
    parsePrimaryExpression(): TwingNodeExpression;
    getFunctionNode(name: string, line: number, column: number): TwingNodeExpression;
    parseStringExpression(): TwingNodeExpression;
    parseArrayExpression(): TwingNodeExpression;
    parseHashExpression(): TwingNodeExpression;
    parseSubscriptExpression(node: TwingNodeExpression): any;
    parsePostfixExpression(node: TwingNodeExpression): TwingNodeExpression;
    parseTestExpression(node: TwingNodeExpression): TwingNodeExpression;
    parseNotTestExpression(node: TwingNodeExpression): TwingNodeExpression;
    parseConditionalExpression(expr: TwingNodeExpression): TwingNodeExpression;
    parseFilterExpression(node: TwingNodeExpression): TwingNodeExpression;
    parseFilterExpressionRaw(node: TwingNodeExpression, tag?: string): TwingNodeExpression;
    /**
     * Parses arguments.
     *
     * @param namedArguments boolean Whether to allow named arguments or not
     * @param definition     boolean Whether we are parsing arguments for a function definition
     *
     * @return TwingNode
     *
     * @throws TwingErrorSyntax
     */
    parseArguments(namedArguments?: boolean, definition?: boolean): TwingNode;
    parseAssignmentExpression(): TwingNode;
    parseMultitargetExpression(): TwingNode;
    checkConstantExpression(node: TwingNode): boolean;
    private isUnary;
    private isBinary;
    private getTest;
    private getFunctionExpressionFactory;
    private getFilterExpressionFactory;
}
