import { Mapping } from "source-map";
export declare type Position = {
    line: number;
    column: number;
};
export declare class TwingSourceMapNode {
    protected name: string;
    protected source: string;
    protected originalPosition: Position;
    protected parent: TwingSourceMapNode;
    protected children: TwingSourceMapNode[];
    protected content: string;
    protected cursor: Position;
    /**
     * TwingSourceMapTreeNode constructor
     *
     * @param {string} type
     * @param {string} source
     * @param {number} line 0-based
     * @param {number} column 0-based
     */
    constructor(name: string, source: string, line: number, column: number);
    getName(): string;
    getSource(): string;
    getOriginalPosition(): Position;
    getContent(): string;
    getParent(): TwingSourceMapNode;
    getChildren(): TwingSourceMapNode[];
    getCursor(): Position;
    addChild(child: TwingSourceMapNode): void;
    setContent(content: string | false): void;
    /**
     * Returns the mappings starting from this node.
     *
     * @returns {Array<Mapping>}
     */
    toMappings(): Array<Mapping>;
}
