import { TwingToken } from "./token";
import { TwingSource } from "./source";
export declare class TwingTokenStream {
    tokens: Array<TwingToken>;
    current: number;
    source: TwingSource;
    constructor(tokens: Array<TwingToken>, source?: TwingSource);
    toString(): string;
    injectTokens(tokens: Array<TwingToken>): void;
    /**
     * Sets the pointer to the next token and returns the old one.
     *
     * @return TwingToken
     */
    next(): TwingToken;
    /**
     * Tests a token, sets the pointer to the next one and returns it or throws a syntax error.
     *
     * @return TwingToken|null The next token if the condition is true, null otherwise
     */
    nextIf(primary: number, secondary?: Array<string> | string): TwingToken;
    /**
     * Tests a token and returns it or throws a syntax error.
     *
     * @return TwingToken
     */
    expect(type: number, value?: Array<string> | string | number, message?: string): TwingToken;
    /**
     * Looks at the next token.
     *
     * @param number {number}
     *
     * @return TwingToken
     */
    look(number?: number): TwingToken;
    /**
     * Tests the active token.
     *
     * @return bool
     */
    test(primary: number, secondary?: Array<string> | string): boolean;
    /**
     * Checks if end of stream was reached.
     *
     * @return bool
     */
    isEOF(): boolean;
    /**
     * @return TwingToken
     */
    getCurrent(): TwingToken;
    /**
     * Gets the source associated with this stream.
     *
     * @return TwingSource
     *
     * @internal
     */
    getSourceContext(): TwingSource;
}
