export declare class TwingToken {
    static EOF_TYPE: number;
    static TEXT_TYPE: number;
    static BLOCK_START_TYPE: number;
    static VAR_START_TYPE: number;
    static BLOCK_END_TYPE: number;
    static VAR_END_TYPE: number;
    static NAME_TYPE: number;
    static NUMBER_TYPE: number;
    static STRING_TYPE: number;
    static OPERATOR_TYPE: number;
    static PUNCTUATION_TYPE: number;
    static INTERPOLATION_START_TYPE: number;
    static INTERPOLATION_END_TYPE: number;
    private value;
    private type;
    private lineno;
    private columnno;
    constructor(type: number, value: string, lineno: number, columnno: number);
    /**
     * Returns the constant representation (internal) of a given type.
     *
     * @param {int} type The type as an integer
     * @param {boolean} short Whether to return a short representation or not
     *
     * @returns {string} The string representation
     */
    static typeToString(type: number, short?: boolean): string;
    /**
     * Returns the English representation of a given type.
     *
     * @param {int} type The type as an integer
     *
     * @returns {string} The string representation
     */
    static typeToEnglish(type: number): string;
    /**
     * Tests the current token for a type and/or a value.
     *
     * Parameters may be:
     *  * just type
     *  * type and value (or array of possible values)
     *  * just value (or array of possible values) (NAME_TYPE is used as type)
     *
     * @param {number} type
     * @param {string|Array<string>} values
     * @returns {boolean}
     */
    test(type: number, values?: Array<string> | string | number): boolean;
    /**
     * @return int
     */
    getLine(): number;
    /**
     * @return int
     */
    getColumn(): number;
    getType(): number;
    getValue(): string;
    toString(): string;
}
